@@ -40,35 +40,33 @@ This also provides a script that can generate docstring definitions from a stub
 file for the C extension itself (drgndoc.docstrings).
 """
 
+import os.path
+import re
+from typing import Any, Dict, List, cast
+
 import docutils.nodes
 import docutils.parsers.rst.directives
 import docutils.statemachine
-import os.path
-import re
 import sphinx.addnodes
 import sphinx.application
 import sphinx.environment
 import sphinx.util.docutils
 import sphinx.util.logging
 import sphinx.util.nodes
-from typing import List, cast
 
 from drgndoc.format import Formatter
 from drgndoc.namespace import Namespace, ResolvedNode
 from drgndoc.parse import (
     Class,
     DocumentedNode,
-    Function,
     Import,
     ImportFrom,
     Module,
     Node,
-    Variable,
     parse_paths,
 )
 from drgndoc.util import dot_join
 
-
 logger = sphinx.util.logging.getLogger(__name__)
 
 
@@ -101,11 +99,10 @@ class DrgnDocDirective(sphinx.util.docutils.SphinxDirective):
     required_arguments = 1
     optional_arguments = 0
     option_spec = {
-        "include": docutils.parsers.rst.directives.unchanged,
         "exclude": docutils.parsers.rst.directives.unchanged,
     }
 
-    def run(self) -> List[docutils.nodes.Node]:
+    def run(self) -> Any:
         parts = []
         py_module = self.env.ref_context.get("py:module")
         if py_module:
@@ -119,98 +116,66 @@ class DrgnDocDirective(sphinx.util.docutils.SphinxDirective):
         if not isinstance(resolved, ResolvedNode):
             logger.warning("name %r not found", resolved)
             return []
+        if not resolved.node.has_docstring():
+            logger.warning("name %r is not documented", resolved.qualified_name())
+            return []
 
         docnode = docutils.nodes.section()
-        self._run(name, "", resolved, docnode)
+        self._run(name, "", self.arguments[0], resolved, docnode)
         return docnode.children
 
-    def _include_attr(self, attr: ResolvedNode[Node], attr_name: str) -> bool:
-        """
-        Return whether the given recursive attribute should be documented.
-
-        We recursively include nodes that are:
-        1. Not imports.
-        2. Match the "include" pattern OR don't start with an underscore.
-        AND
-        3. Do not match the "exclude" pattern.
-
-        The "include" and "exclude" patterns are applied to the name relative
-        to the object being documented by the directive.
-        """
-        if isinstance(attr.node, (Import, ImportFrom)):
-            return False
-
-        if not attr_name:
-            return True
-
-        dot = attr_name.rfind(".")
-        if dot + 1 < len(attr_name) and attr_name[dot + 1] == "_":
-            include_pattern = self.options.get("include")
-            if include_pattern is None or not re.fullmatch(include_pattern, attr_name):
-                return False
-        exclude_pattern = self.options.get("exclude")
-        return exclude_pattern is None or not re.fullmatch(exclude_pattern, attr_name)
-
     def _run(
         self,
         top_name: str,
         attr_name: str,
+        name: str,
         resolved: ResolvedNode[Node],
         docnode: docutils.nodes.Node,
     ) -> None:
-        if not self._include_attr(resolved, attr_name):
+        exclude_pattern = self.options.get("exclude")
+        if exclude_pattern is not None and re.fullmatch(exclude_pattern, attr_name):
             return
+
+        if isinstance(resolved.node, (Import, ImportFrom)):
+            # Only include imports that are explicitly aliased (i.e., import
+            # ... as ... or from ... import ... as ...).
+            # TODO: we should also include imports listed in __all__.
+            if not resolved.node.aliased:
+                return
+            imported = self.env.drgndoc_namespace.resolve_name_in_scope(
+                resolved.modules, resolved.classes, resolved.name
+            )
+            if not isinstance(imported, ResolvedNode):
+                return
+            resolved = imported
+
         resolved = cast(ResolvedNode[DocumentedNode], resolved)
 
-        node = resolved.node
-        if isinstance(node, Module):
-            directive = "py:module"
+        if isinstance(resolved.node, Module):
             return self._run_module(
                 top_name, attr_name, cast(ResolvedNode[Module], resolved), docnode
             )
 
-        sourcename = ""
-        if resolved.module and resolved.module.node.path:
-            sourcename = resolved.module.node.path
-        if sourcename:
-            self.env.note_dependency(sourcename)
-
-        if isinstance(node, Class):
-            directive = "py:class"
-        elif isinstance(node, Function):
-            directive = "py:method" if resolved.class_ else "py:function"
-        elif isinstance(node, Variable):
-            directive = "py:attribute" if resolved.class_ else "py:data"
-        else:
-            assert False, type(node).__name__
-
-        argument = (attr_name or top_name).rpartition(".")[2]
-        extra_argument, lines = self.env.drgndoc_formatter.format(
+        lines = self.env.drgndoc_formatter.format(
             resolved,
+            name,
             self.env.ref_context.get("py:module", ""),
             ".".join(self.env.ref_context.get("py:classes", ())),
         )
+        if not lines:
+            # Not documented. Ignore it.
+            return
 
-        contents = docutils.statemachine.StringList()
-        contents.append(
-            f".. {directive}:: {argument}{extra_argument}", sourcename,
-        )
-        if isinstance(node, Function):
-            if node.async_:
-                contents.append("    :async:", sourcename)
-            if resolved.class_:
-                if node.have_decorator("classmethod"):
-                    contents.append("    :classmethod:", sourcename)
-                if node.have_decorator("staticmethod"):
-                    contents.append("    :staticmethod:", sourcename)
+        sourcename = ""
+        if resolved.modules and resolved.modules[-1].node.path:
+            sourcename = resolved.modules[-1].node.path
+        if sourcename:
+            self.env.note_dependency(sourcename)
+        contents = docutils.statemachine.StringList(lines, sourcename)
         contents.append("", sourcename)
-        if lines:
-            for line in lines:
-                contents.append("    " + line, sourcename)
-            contents.append("", sourcename)
 
         self.state.nested_parse(contents, 0, docnode)
-        if isinstance(node, Class):
+        if isinstance(resolved.node, Class):
             for desc in reversed(docnode.children):
                 if isinstance(desc, sphinx.addnodes.desc):
                     break
@@ -228,9 +193,14 @@ class DrgnDocDirective(sphinx.util.docutils.SphinxDirective):
             py_classes.append(resolved.name)
             self.env.ref_context["py:class"] = resolved.name
             for member in resolved.attrs():
-                self._run(
-                    top_name, dot_join(attr_name, member.name), member, desc_content
-                )
+                if member.name != "__init__":
+                    self._run(
+                        top_name,
+                        dot_join(attr_name, member.name),
+                        member.name,
+                        member,
+                        desc_content,
+                    )
             py_classes.pop()
             self.env.ref_context["py:class"] = py_classes[-1] if py_classes else None
 
@@ -242,14 +212,16 @@ class DrgnDocDirective(sphinx.util.docutils.SphinxDirective):
         docnode: docutils.nodes.Node,
     ) -> None:
         node = resolved.node
+        if node.docstring is None:
+            # Not documented. Ignore it.
+            return
+
         sourcename = node.path or ""
         if sourcename:
             self.env.note_dependency(sourcename)
-
-        contents = docutils.statemachine.StringList()
-        if node.docstring:
-            for line in node.docstring.splitlines():
-                contents.append(line, sourcename)
+        contents = docutils.statemachine.StringList(
+            node.docstring.splitlines(), sourcename
+        )
 
         sphinx.util.nodes.nested_parse_with_titles(self.state, contents, docnode)
 
@@ -268,14 +240,16 @@ class DrgnDocDirective(sphinx.util.docutils.SphinxDirective):
             have_old_py_module = False
         self.env.ref_context["py:module"] = dot_join(top_name, attr_name)
         for attr in resolved.attrs():
-            self._run(top_name, dot_join(attr_name, attr.name), attr, section)
+            self._run(
+                top_name, dot_join(attr_name, attr.name), attr.name, attr, section
+            )
         if have_old_py_module:
             self.env.ref_context["py:module"] = old_py_module
         else:
             del self.env.ref_context["py:module"]
 
 
-def setup(app: sphinx.application.Sphinx) -> dict:
+def setup(app: sphinx.application.Sphinx) -> Dict[str, Any]:
     app.connect("builder-inited", drgndoc_init)
     # List of modules or packages.
     app.add_config_value("drgndoc_paths", [], "env")
