@@ -36,16 +36,28 @@ def for_each_cpu(mask: Object) -> Iterator[int]:
                 yield (word_bits * i) + j
 
 
-def for_each_possible_cpu(prog: Program) -> Iterator[int]:
-    """Iterate over all possible CPUs."""
-    return for_each_cpu(prog["__cpu_possible_mask"])
+def _for_each_cpu_mask(prog: Program, name: str) -> Iterator[int]:
+    try:
+        mask = prog[name]
+    except KeyError:
+        # Before Linux kernel commit c4c54dd1caf1 ("kernel/cpu.c: change type
+        # of cpu_possible_bits and friends") (in v4.5), the CPU masks are
+        # struct cpumask *cpu_foo_mask instead of
+        # struct cpumask __cpu_foo_mask.
+        mask = prog[name[2:]][0]
+    return for_each_cpu(mask)
 
 
 def for_each_online_cpu(prog: Program) -> Iterator[int]:
     """Iterate over all online CPUs."""
-    return for_each_cpu(prog["__cpu_online_mask"])
+    return _for_each_cpu_mask(prog, "__cpu_online_mask")
+
+
+def for_each_possible_cpu(prog: Program) -> Iterator[int]:
+    """Iterate over all possible CPUs."""
+    return _for_each_cpu_mask(prog, "__cpu_possible_mask")
 
 
 def for_each_present_cpu(prog: Program) -> Iterator[int]:
     """Iterate over all present CPUs."""
-    return for_each_cpu(prog["__cpu_present_mask"])
+    return _for_each_cpu_mask(prog, "__cpu_present_mask")
