@@ -10,8 +10,9 @@ in the Linux kernel.
 """
 
 import operator
+from typing import Iterator, Union
 
-from drgn import NULL, Object
+from drgn import NULL, IntegerLike, Object, Program
 from drgn.helpers.linux.list import hlist_for_each_entry
 
 __all__ = (
@@ -20,18 +21,18 @@ __all__ = (
 )
 
 
-def _kuid_val(uid):
+def _kuid_val(uid: Union[Object, IntegerLike]) -> int:
     if isinstance(uid, Object) and uid.type_.type_name() == "kuid_t":
         uid = uid.val
     return operator.index(uid)
 
 
-def find_user(prog, uid):
+def find_user(prog: Program, uid: Union[Object, IntegerLike]) -> Object:
     """
-    .. c:function:: struct user_struct *find_user(kuid_t uid)
+    Return the user structure with the given UID.
 
-    Return the user structure with the given UID, which may be a ``kuid_t`` or
-    an integer.
+    :param uid: ``kuid_t`` object or integer.
+    :return: ``struct user_state *``
     """
     try:
         uidhashentry = prog.cache["uidhashentry"]
@@ -41,7 +42,7 @@ def find_user(prog, uid):
         uidhash_bits = uidhash_sz.bit_length() - 1
         uidhash_mask = uidhash_sz - 1
 
-        def uidhashentry(uid):
+        def uidhashentry(uid: int) -> Object:
             hash = ((uid >> uidhash_bits) + uid) & uidhash_mask
             return uidhash_table + hash
 
@@ -56,7 +57,7 @@ def find_user(prog, uid):
     return NULL(prog, "struct user_struct *")
 
 
-def for_each_user(prog):
+def for_each_user(prog: Program) -> Iterator[Object]:
     """
     Iterate over all users in the system.
 
