@@ -337,7 +337,7 @@ c_declare_variable(struct drgn_qualified_type qualified_type,
 		   struct string_callback *name, size_t indent,
 		   struct string_builder *sb)
 {
-	switch (drgn_type_kind(qualified_type.type)) {
+	SWITCH_ENUM(drgn_type_kind(qualified_type.type),
 	case DRGN_TYPE_VOID:
 	case DRGN_TYPE_INT:
 	case DRGN_TYPE_BOOL:
@@ -356,8 +356,7 @@ c_declare_variable(struct drgn_qualified_type qualified_type,
 		return c_declare_array(qualified_type, name, indent, sb);
 	case DRGN_TYPE_FUNCTION:
 		return c_declare_function(qualified_type, name, indent, sb);
-	}
-	UNREACHABLE();
+	)
 }
 
 static struct drgn_error *
@@ -383,27 +382,27 @@ c_define_compound(struct drgn_qualified_type qualified_type, size_t indent,
 		return &drgn_enomem;
 
 	for (i = 0; i < num_members; i++) {
-		const char *member_name = members[i].name;
 		struct drgn_qualified_type member_type;
+		uint64_t member_bit_field_size;
+		err = drgn_member_type(&members[i], &member_type,
+				       &member_bit_field_size);
+		if (err)
+			return err;
+
+		const char *member_name = members[i].name;
 		struct string_callback name_cb = {
 			.fn = c_variable_name,
 			.arg = (void *)member_name,
 		};
-
-		err = drgn_member_type(&members[i], &member_type);
-		if (err)
-			return err;
-
 		err = c_declare_variable(member_type,
 					 member_name && member_name[0] ?
 					 &name_cb : NULL, indent + 1, sb);
 		if (err)
 			return err;
-		if (members[i].bit_field_size) {
-			if (!string_builder_appendf(sb, " : %" PRIu64,
-						    members[i].bit_field_size))
+		if (member_bit_field_size &&
+		    !string_builder_appendf(sb, " : %" PRIu64,
+					    member_bit_field_size))
 				return &drgn_enomem;
-		}
 		if (!string_builder_append(sb, ";\n"))
 			return &drgn_enomem;
 	}
@@ -489,7 +488,7 @@ static struct drgn_error *
 c_define_type(struct drgn_qualified_type qualified_type, size_t indent,
 	      struct string_builder *sb)
 {
-	switch (drgn_type_kind(qualified_type.type)) {
+	SWITCH_ENUM(drgn_type_kind(qualified_type.type),
 	case DRGN_TYPE_VOID:
 	case DRGN_TYPE_INT:
 	case DRGN_TYPE_BOOL:
@@ -511,8 +510,7 @@ c_define_type(struct drgn_qualified_type qualified_type, size_t indent,
 	case DRGN_TYPE_FUNCTION:
 		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
 					 "function type cannot be formatted");
-	}
-	UNREACHABLE();
+	)
 }
 
 static struct drgn_error *
@@ -708,8 +706,8 @@ c_format_int_object(const struct drgn_object *obj,
 		return NULL;
 	}
 
-	switch (obj->kind) {
-	case DRGN_OBJECT_SIGNED: {
+	switch (obj->encoding) {
+	case DRGN_OBJECT_ENCODING_SIGNED: {
 		int64_t svalue;
 
 		err = drgn_object_read_signed(obj, &svalue);
@@ -719,7 +717,7 @@ c_format_int_object(const struct drgn_object *obj,
 			return &drgn_enomem;
 		return NULL;
 	}
-	case DRGN_OBJECT_UNSIGNED: {
+	case DRGN_OBJECT_ENCODING_UNSIGNED: {
 		uint64_t uvalue;
 
 		err = drgn_object_read_unsigned(obj, &uvalue);
@@ -1011,26 +1009,24 @@ compound_initializer_iter_next(struct initializer_iter *iter_,
 	struct drgn_error *err;
 	struct compound_initializer_iter *iter =
 		container_of(iter_, struct compound_initializer_iter, iter);
-	struct compound_initializer_state *top;
-	uint64_t bit_offset;
-	struct drgn_type_member *member;
-	struct drgn_qualified_type member_type;
 
 	for (;;) {
-		struct compound_initializer_state *new;
-
 		if (!iter->stack.size)
 			return &drgn_stop;
 
-		top = &iter->stack.data[iter->stack.size - 1];
+		struct compound_initializer_state *top =
+			&iter->stack.data[iter->stack.size - 1];
 		if (top->member == top->end) {
 			iter->stack.size--;
 			continue;
 		}
 
-		bit_offset = top->bit_offset;
-		member = top->member++;
-		err = drgn_member_type(member, &member_type);
+		uint64_t bit_offset = top->bit_offset;
+		struct drgn_type_member *member = top->member++;
+		struct drgn_qualified_type member_type;
+		uint64_t member_bit_field_size;
+		err = drgn_member_type(member, &member_type,
+				       &member_bit_field_size);
 		if (err)
 			return err;
 
@@ -1045,7 +1041,7 @@ compound_initializer_iter_next(struct initializer_iter *iter_,
 		    !drgn_type_has_members(member_type.type)) {
 			err = drgn_object_slice(obj_ret, iter->obj, member_type,
 						bit_offset + member->bit_offset,
-						member->bit_field_size);
+						member_bit_field_size);
 			if (err)
 				return err;
 
@@ -1064,7 +1060,8 @@ compound_initializer_iter_next(struct initializer_iter *iter_,
 			break;
 		}
 
-		new = compound_initializer_stack_append_entry(&iter->stack);
+		struct compound_initializer_state *new =
+			compound_initializer_stack_append_entry(&iter->stack);
 		if (!new)
 			return &drgn_enomem;
 		new->member = drgn_type_members(member_type.type);
@@ -1110,19 +1107,6 @@ c_format_compound_object(const struct drgn_object *obj,
 			 struct string_builder *sb)
 {
 	struct drgn_error *err;
-	struct compound_initializer_iter iter = {
-		.iter = {
-			.next = compound_initializer_iter_next,
-			.reset = compound_initializer_iter_reset,
-			.append_designation =
-				flags & DRGN_FORMAT_OBJECT_MEMBER_NAMES ?
-				compound_initializer_append_designation : NULL,
-		},
-		.obj = obj,
-		.flags = flags,
-		.member_flags = drgn_member_format_object_flags(flags),
-	};
-	struct compound_initializer_state *new;
 
 	if (!drgn_type_is_complete(underlying_type)) {
 		const char *keyword;
@@ -1145,8 +1129,21 @@ c_format_compound_object(const struct drgn_object *obj,
 					 keyword);
 	}
 
-	compound_initializer_stack_init(&iter.stack);
-	new = compound_initializer_stack_append_entry(&iter.stack);
+	struct compound_initializer_iter iter = {
+		.iter = {
+			.next = compound_initializer_iter_next,
+			.reset = compound_initializer_iter_reset,
+			.append_designation =
+				flags & DRGN_FORMAT_OBJECT_MEMBER_NAMES ?
+				compound_initializer_append_designation : NULL,
+		},
+		.obj = obj,
+		.stack = VECTOR_INIT,
+		.flags = flags,
+		.member_flags = drgn_member_format_object_flags(flags),
+	};
+	struct compound_initializer_state *new =
+		compound_initializer_stack_append_entry(&iter.stack);
 	if (!new) {
 		err = &drgn_enomem;
 		goto out;
@@ -1164,22 +1161,22 @@ c_format_compound_object(const struct drgn_object *obj,
 		       DRGN_FORMAT_OBJECT_IMPLICIT_MEMBERS)) &&
 	    new->member < new->end) {
 		struct drgn_object member;
-
 		drgn_object_init(&member, drgn_object_program(obj));
 		do {
 			struct drgn_qualified_type member_type;
-			bool zero;
-
-			err = drgn_member_type(&new->end[-1], &member_type);
+			uint64_t member_bit_field_size;
+			err = drgn_member_type(&new->end[-1], &member_type,
+					       &member_bit_field_size);
 			if (err)
 				break;
 
 			err = drgn_object_slice(&member, obj, member_type,
 						new->end[-1].bit_offset,
-						new->end[-1].bit_field_size);
+						member_bit_field_size);
 			if (err)
 				break;
 
+			bool zero;
 			err = drgn_object_is_zero(&member, &zero);
 			if (err)
 				break;
@@ -1456,18 +1453,15 @@ c_format_array_object(const struct drgn_object *obj,
 
 	if ((flags & DRGN_FORMAT_OBJECT_STRING) && iter.length &&
 	    is_character_type(iter.element_type.type)) {
-		if (obj->is_reference) {
-			return c_format_string(&drgn_object_program(obj)->reader,
-					       obj->reference.address,
-					       iter.length, sb);
-		} else {
+		SWITCH_ENUM(obj->kind,
+		case DRGN_OBJECT_VALUE: {
 			const unsigned char *buf;
 			uint64_t size, i;
 
 			if (!string_builder_appendc(sb, '"'))
 				return &drgn_enomem;
 			buf = (const unsigned char *)drgn_object_buffer(obj);
-			size = drgn_buffer_object_size(obj);
+			size = drgn_object_size(obj);
 			for (i = 0; i < size; i++) {
 				if (buf[i] == '\0')
 					break;
@@ -1480,6 +1474,11 @@ c_format_array_object(const struct drgn_object *obj,
 				return &drgn_enomem;
 			return NULL;
 		}
+		case DRGN_OBJECT_REFERENCE:
+			return c_format_string(&drgn_object_program(obj)->reader,
+					       obj->address, iter.length, sb);
+		case DRGN_OBJECT_ABSENT:
+		)
 	}
 
 	err = drgn_type_bit_size(iter.element_type.type,
@@ -1532,12 +1531,8 @@ static struct drgn_error *
 c_format_function_object(const struct drgn_object *obj,
 			 struct string_builder *sb)
 {
-	/* Function values currently aren't possible anyways. */
-	if (!obj->is_reference) {
-		return drgn_error_create(DRGN_ERROR_TYPE,
-					 "cannot format function value");
-	}
-	if (!string_builder_appendf(sb, "0x%" PRIx64, obj->reference.address))
+	assert(obj->kind == DRGN_OBJECT_REFERENCE);
+	if (!string_builder_appendf(sb, "0x%" PRIx64, obj->address))
 		return &drgn_enomem;
 	return NULL;
 }
@@ -1551,11 +1546,17 @@ c_format_object_impl(const struct drgn_object *obj, size_t indent,
 	struct drgn_error *err;
 	struct drgn_type *underlying_type = drgn_underlying_type(obj->type);
 
+	if (drgn_type_kind(underlying_type) == DRGN_TYPE_VOID) {
+		return drgn_error_create(DRGN_ERROR_TYPE,
+					 "cannot format void object");
+	}
+
 	/*
 	 * Pointers are special because they can have an asterisk prefix if
 	 * we're dereferencing them.
 	 */
-	if (drgn_type_kind(underlying_type) == DRGN_TYPE_POINTER) {
+	if (drgn_type_kind(underlying_type) == DRGN_TYPE_POINTER &&
+	    obj->kind != DRGN_OBJECT_ABSENT) {
 		return c_format_pointer_object(obj, underlying_type, indent,
 					       one_line_columns,
 					       multi_line_columns, flags, sb);
@@ -1578,10 +1579,13 @@ c_format_object_impl(const struct drgn_object *obj, size_t indent,
 		    one_line_columns = 0;
 	}
 
-	switch (drgn_type_kind(underlying_type)) {
-	case DRGN_TYPE_VOID:
-		return drgn_error_create(DRGN_ERROR_TYPE,
-					 "cannot format void object");
+	if (obj->kind == DRGN_OBJECT_ABSENT) {
+		if (!string_builder_append(sb, "<absent>"))
+			return &drgn_enomem;
+		return NULL;
+	}
+
+	SWITCH_ENUM(drgn_type_kind(underlying_type),
 	case DRGN_TYPE_INT:
 	case DRGN_TYPE_BOOL:
 		return c_format_int_object(obj, flags, sb);
@@ -1604,9 +1608,10 @@ c_format_object_impl(const struct drgn_object *obj, size_t indent,
 					     multi_line_columns, flags, sb);
 	case DRGN_TYPE_FUNCTION:
 		return c_format_function_object(obj, sb);
-	default:
-		UNREACHABLE();
-	}
+	case DRGN_TYPE_VOID:
+	case DRGN_TYPE_TYPEDEF:
+	case DRGN_TYPE_POINTER:
+	)
 }
 
 struct drgn_error *c_format_object(const struct drgn_object *obj,
@@ -2589,24 +2594,25 @@ struct drgn_error *c_bit_offset(struct drgn_program *prog,
 		case INT_MIN:
 		case C_TOKEN_DOT:
 			if (token.kind == C_TOKEN_IDENTIFIER) {
-				struct drgn_member_value *member;
-				struct drgn_qualified_type member_type;
-
-				err = drgn_program_find_member(prog, type,
-							       token.value,
-							       token.len,
-							       &member);
+				struct drgn_type_member *member;
+				uint64_t member_bit_offset;
+				err = drgn_type_find_member_len(type,
+								token.value,
+								token.len,
+								&member,
+								&member_bit_offset);
 				if (err)
 					goto out;
 				if (__builtin_add_overflow(bit_offset,
-							   member->bit_offset,
+							   member_bit_offset,
 							   &bit_offset)) {
 					err = drgn_error_create(DRGN_ERROR_OVERFLOW,
 								"offset is too large");
 					goto out;
 				}
-				err = drgn_lazy_type_evaluate(member->type,
-							      &member_type);
+				struct drgn_qualified_type member_type;
+				err = drgn_member_type(member, &member_type,
+						       NULL);
 				if (err)
 					goto out;
 				type = member_type.type;
