@@ -8,7 +8,25 @@
 #include "drgnpy.h"
 #include "../path.h"
 
+/* Basically PyModule_AddType(), which is only available since Python 3.9. */
+static int add_type(PyObject *module, PyTypeObject *type)
+{
+	int ret = PyType_Ready(type);
+	if (ret)
+		return ret;
+	const char *name = type->tp_name;
+	const char *dot = strrchr(type->tp_name, '.');
+	if (dot)
+		name = dot + 1;
+	Py_INCREF(type);
+	ret = PyModule_AddObject(module, name, (PyObject *)type);
+	if (ret)
+		Py_DECREF(type);
+	return ret;
+}
+
 PyObject *MissingDebugInfoError;
+PyObject *ObjectAbsentError;
 PyObject *OutOfBoundsError;
 
 static PyObject *filename_matches(PyObject *self, PyObject *args,
@@ -17,39 +35,34 @@ static PyObject *filename_matches(PyObject *self, PyObject *args,
 	static char *keywords[] = {"haystack", "needle", NULL};
 	struct path_arg haystack_arg = {.allow_none = true};
 	struct path_arg needle_arg = {.allow_none = true};
-	struct path_iterator haystack = {
-		.components = (struct path_iterator_component [1]){},
-		.num_components = 0,
-	};
-	struct path_iterator needle = {
-		.components = (struct path_iterator_component [1]){},
-		.num_components = 0,
-	};
-	bool ret;
-
 	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O&O&:filename_matches",
 					 keywords, path_converter,
 					 &haystack_arg, path_converter,
 					 &needle_arg))
 		return NULL;
 
+	struct path_iterator haystack = {
+		.components = (struct path_iterator_component [1]){},
+		.num_components = 0,
+	};
 	if (haystack_arg.path) {
 		haystack.components[0].path = haystack_arg.path;
 		haystack.components[0].len = haystack_arg.length;
 		haystack.num_components = 1;
 	}
+	struct path_iterator needle = {
+		.components = (struct path_iterator_component [1]){},
+		.num_components = 0,
+	};
 	if (needle_arg.path) {
 		needle.components[0].path = needle_arg.path;
 		needle.components[0].len = needle_arg.length;
 		needle.num_components = 1;
 	}
-	ret = path_ends_with(&haystack, &needle);
+	bool ret = path_ends_with(&haystack, &needle);
 	path_cleanup(&haystack_arg);
 	path_cleanup(&needle_arg);
-	if (ret)
-		Py_RETURN_TRUE;
-	else
-		Py_RETURN_FALSE;
+	Py_RETURN_BOOL(ret);
 }
 
 static PyObject *sizeof_(PyObject *self, PyObject *arg)
@@ -71,12 +84,31 @@ static PyObject *sizeof_(PyObject *self, PyObject *arg)
 	return PyLong_FromUnsignedLongLong(size);
 }
 
+static PyObject *offsetof_(PyObject *self, PyObject *args, PyObject *kwds)
+{
+	struct drgn_error *err;
+
+	static char *keywords[] = {"type", "member", NULL};
+	DrgnType *type;
+	const char *member;
+	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!s:offsetof", keywords,
+					 &DrgnType_type, &type, &member))
+		return NULL;
+	uint64_t offset;
+	err = drgn_type_offsetof(type->type, member, &offset);
+	if (err)
+		return set_drgn_error(err);
+	return PyLong_FromUnsignedLongLong(offset);
+}
+
 static PyMethodDef drgn_methods[] = {
 	{"filename_matches", (PyCFunction)filename_matches,
 	 METH_VARARGS | METH_KEYWORDS, drgn_filename_matches_DOC},
 	{"NULL", (PyCFunction)DrgnObject_NULL, METH_VARARGS | METH_KEYWORDS,
 	 drgn_NULL_DOC},
 	{"sizeof", (PyCFunction)sizeof_, METH_O, drgn_sizeof_DOC},
+	{"offsetof", (PyCFunction)offsetof_, METH_VARARGS | METH_KEYWORDS,
+	 drgn_offsetof_DOC},
 	{"cast", (PyCFunction)cast, METH_VARARGS | METH_KEYWORDS,
 	 drgn_cast_DOC},
 	{"reinterpret", (PyCFunction)reinterpret, METH_VARARGS | METH_KEYWORDS,
@@ -181,114 +213,84 @@ static int add_type_aliases(PyObject *m)
 
 DRGNPY_PUBLIC PyMODINIT_FUNC PyInit__drgn(void)
 {
-	PyObject *m;
-	PyObject *host_platform_obj;
-	PyObject *with_libkdumpfile;
-
-	m = PyModule_Create(&drgnmodule);
+	PyObject *m = PyModule_Create(&drgnmodule);
 	if (!m)
 		return NULL;
 
-	if (add_module_constants(m) == -1 || add_type_aliases(m) == -1)
+	if (add_module_constants(m) ||
+	    add_type_aliases(m) ||
+	    add_type(m, &Language_type) || add_languages() ||
+	    add_type(m, &DrgnObject_type) ||
+	    PyType_Ready(&ObjectIterator_type) ||
+	    add_type(m, &Platform_type) ||
+	    add_type(m, &Program_type) ||
+	    add_type(m, &Register_type) ||
+	    add_type(m, &StackFrame_type) ||
+	    add_type(m, &StackTrace_type) ||
+	    add_type(m, &Symbol_type) ||
+	    add_type(m, &DrgnType_type) ||
+	    add_type(m, &TypeEnumerator_type) ||
+	    add_type(m, &TypeMember_type) ||
+	    add_type(m, &TypeParameter_type) ||
+	    add_type(m, &TypeTemplateParameter_type))
 		goto err;
 
 	FaultError_type.tp_base = (PyTypeObject *)PyExc_Exception;
-	if (PyType_Ready(&FaultError_type) < 0)
+	if (add_type(m, &FaultError_type))
 		goto err;
-	Py_INCREF(&FaultError_type);
-	PyModule_AddObject(m, "FaultError", (PyObject *)&FaultError_type);
 
-	MissingDebugInfoError = PyErr_NewExceptionWithDoc("_drgn.MissingDebugInfoError",
-							  drgn_MissingDebugInfoError_DOC,
-							  NULL, NULL);
+	MissingDebugInfoError =
+		PyErr_NewExceptionWithDoc("_drgn.MissingDebugInfoError",
+					  drgn_MissingDebugInfoError_DOC, NULL,
+					  NULL);
 	if (!MissingDebugInfoError)
 		goto err;
-	PyModule_AddObject(m, "MissingDebugInfoError", MissingDebugInfoError);
+	if (PyModule_AddObject(m, "MissingDebugInfoError",
+			       MissingDebugInfoError)) {
+		Py_DECREF(MissingDebugInfoError);
+		goto err;
+	}
+
+	ObjectAbsentError =
+		PyErr_NewExceptionWithDoc("_drgn.ObjectAbsentError",
+					  drgn_ObjectAbsentError_DOC,
+					  NULL, NULL);
+	if (!ObjectAbsentError)
+		goto err;
+	if (PyModule_AddObject(m, "ObjectAbsentError", ObjectAbsentError)) {
+		Py_DECREF(ObjectAbsentError);
+		goto err;
+	}
 
 	OutOfBoundsError = PyErr_NewExceptionWithDoc("_drgn.OutOfBoundsError",
 						     drgn_OutOfBoundsError_DOC,
 						     NULL, NULL);
 	if (!OutOfBoundsError)
 		goto err;
-	PyModule_AddObject(m, "OutOfBoundsError", OutOfBoundsError);
-
-	if (PyType_Ready(&Language_type) < 0)
-		goto err;
-	Py_INCREF(&Language_type);
-	PyModule_AddObject(m, "Language", (PyObject *)&Language_type);
-	if (add_languages() == -1)
+	if (PyModule_AddObject(m, "OutOfBoundsError", OutOfBoundsError)) {
+		Py_DECREF(OutOfBoundsError);
 		goto err;
+	}
 
-	if (PyStructSequence_InitType2(&Register_type, &Register_desc) == -1)
-		goto err;
-	PyModule_AddObject(m, "Register", (PyObject *)&Register_type);
-
-	if (PyType_Ready(&DrgnObject_type) < 0)
-		goto err;
-	Py_INCREF(&DrgnObject_type);
-	PyModule_AddObject(m, "Object", (PyObject *)&DrgnObject_type);
-
-	if (PyType_Ready(&ObjectIterator_type) < 0)
-		goto err;
-
-	if (PyType_Ready(&Platform_type) < 0)
-		goto err;
-	Py_INCREF(&Platform_type);
-	PyModule_AddObject(m, "Platform", (PyObject *)&Platform_type);
-
-	if (PyType_Ready(&Program_type) < 0)
-		goto err;
-	Py_INCREF(&Program_type);
-	PyModule_AddObject(m, "Program", (PyObject *)&Program_type);
-
-	if (PyType_Ready(&StackFrame_type) < 0)
-		goto err;
-	Py_INCREF(&StackFrame_type);
-	PyModule_AddObject(m, "StackFrame", (PyObject *)&StackFrame_type);
-
-	if (PyType_Ready(&StackTrace_type) < 0)
-		goto err;
-	Py_INCREF(&StackTrace_type);
-	PyModule_AddObject(m, "StackTrace", (PyObject *)&StackTrace_type);
-
-	if (PyType_Ready(&Symbol_type) < 0)
-		goto err;
-	Py_INCREF(&Symbol_type);
-	PyModule_AddObject(m, "Symbol", (PyObject *)&Symbol_type);
-
-	if (PyType_Ready(&DrgnType_type) < 0)
-		goto err;
-	Py_INCREF(&DrgnType_type);
-	PyModule_AddObject(m, "Type", (PyObject *)&DrgnType_type);
-
-	if (PyType_Ready(&TypeEnumerator_type) < 0)
-		goto err;
-	Py_INCREF(&TypeEnumerator_type);
-	PyModule_AddObject(m, "TypeEnumerator",
-			   (PyObject *)&TypeEnumerator_type);
-
-	if (PyType_Ready(&TypeMember_type) < 0)
-		goto err;
-	Py_INCREF(&TypeMember_type);
-	PyModule_AddObject(m, "TypeMember", (PyObject *)&TypeMember_type);
-
-	if (PyType_Ready(&TypeParameter_type) < 0)
-		goto err;
-	Py_INCREF(&TypeParameter_type);
-	PyModule_AddObject(m, "TypeParameter", (PyObject *)&TypeParameter_type);
-
-	host_platform_obj = Platform_wrap(&drgn_host_platform);
+	PyObject *host_platform_obj = Platform_wrap(&drgn_host_platform);
 	if (!host_platform_obj)
 		goto err;
-	PyModule_AddObject(m, "host_platform", host_platform_obj);
+	if (PyModule_AddObject(m, "host_platform", host_platform_obj)) {
+		Py_DECREF(host_platform_obj);
+		goto err;
+	}
 
+	PyObject *with_libkdumpfile;
 #ifdef WITH_LIBKDUMPFILE
 	with_libkdumpfile = Py_True;
 #else
 	with_libkdumpfile = Py_False;
 #endif
 	Py_INCREF(with_libkdumpfile);
-	PyModule_AddObject(m, "_with_libkdumpfile", with_libkdumpfile);
+	if (PyModule_AddObject(m, "_with_libkdumpfile", with_libkdumpfile)) {
+		Py_DECREF(with_libkdumpfile);
+		goto err;
+	}
 
 	return m;
 
