@@ -59,18 +59,14 @@ static void Platform_dealloc(Platform *self)
 
 static PyObject *Platform_richcompare(Platform *self, PyObject *other, int op)
 {
-	bool ret;
-
 	if (!PyObject_TypeCheck(other, &Platform_type) ||
 	    (op != Py_EQ && op != Py_NE))
 		Py_RETURN_NOTIMPLEMENTED;
-	ret = drgn_platform_eq(self->platform, ((Platform *)other)->platform);
+	bool ret = drgn_platform_eq(self->platform,
+				    ((Platform *)other)->platform);
 	if (op == Py_NE)
 		ret = !ret;
-	if (ret)
-		Py_RETURN_TRUE;
-	else
-		Py_RETURN_FALSE;
+	Py_RETURN_BOOL(ret);
 }
 
 static PyObject *Platform_get_arch(Platform *self, void *arg)
@@ -87,38 +83,19 @@ static PyObject *Platform_get_flags(Platform *self, void *arg)
 
 static PyObject *Platform_get_registers(Platform *self, void *arg)
 {
-	PyObject *tuple;
-	size_t num_registers, i;
-
-	num_registers = drgn_platform_num_registers(self->platform);
-	tuple = PyTuple_New(num_registers);
+	size_t num_registers = drgn_platform_num_registers(self->platform);
+	PyObject *tuple = PyTuple_New(num_registers);
 	if (!tuple)
 		return NULL;
-	for (i = 0; i < num_registers; i++) {
-		const struct drgn_register *reg;
-		PyObject *item;
-		PyObject *tmp;
-
-		reg = drgn_platform_register(self->platform, i);
-		item = PyStructSequence_New(&Register_type);
+	for (size_t i = 0; i < num_registers; i++) {
+		const struct drgn_register *reg =
+			drgn_platform_register(self->platform, i);
+		PyObject *item = Register_type.tp_alloc(&Register_type, 0);
 		if (!item) {
 			Py_DECREF(tuple);
 			return NULL;
 		}
-		tmp = PyUnicode_FromString(drgn_register_name(reg));
-		if (!tmp) {
-			Py_DECREF(item);
-			Py_DECREF(tuple);
-			return NULL;
-		}
-		PyStructSequence_SET_ITEM(item, 0, tmp);
-		tmp = PyLong_FromLong(drgn_register_number(reg));
-		if (!tmp) {
-			Py_DECREF(item);
-			Py_DECREF(tuple);
-			return NULL;
-		}
-		PyStructSequence_SET_ITEM(item, 1, tmp);
+		((Register *)item)->reg = reg;
 		PyTuple_SET_ITEM(tuple, i, item);
 	}
 	return tuple;
@@ -163,17 +140,55 @@ PyTypeObject Platform_type = {
 	.tp_new = (newfunc)Platform_new,
 };
 
-static PyStructSequence_Field Register_fields[] = {
-	{"name", drgn_Register_name_DOC},
-	{"number", drgn_Register_number_DOC},
+static PyObject *Register_richcompare(Register *self, PyObject *other, int op)
+{
+	if (!PyObject_TypeCheck(other, &Register_type) ||
+	    (op != Py_EQ && op != Py_NE))
+		Py_RETURN_NOTIMPLEMENTED;
+	bool ret = self->reg == ((Register *)other)->reg;
+	if (op == Py_NE)
+		ret = !ret;
+	Py_RETURN_BOOL(ret);
+}
+
+static PyObject *Register_get_names(Register *self, void *arg)
+{
+	size_t num_names;
+	const char * const *names = drgn_register_names(self->reg, &num_names);
+	PyObject *ret = PyTuple_New(num_names);
+	for (size_t i = 0; i < num_names; i++) {
+		PyObject *item = PyUnicode_FromString(names[i]);
+		if (!item) {
+			Py_DECREF(ret);
+			return NULL;
+		}
+		PyTuple_SET_ITEM(ret, i, item);
+	}
+	return ret;
+}
+
+static PyObject *Register_repr(Register *self)
+{
+	PyObject *names_obj = Register_get_names(self, NULL);
+	if (!names_obj)
+		return NULL;
+	PyObject *ret = PyUnicode_FromFormat("Register(%R)", names_obj);
+	Py_DECREF(names_obj);
+	return ret;
+}
+
+static PyGetSetDef Register_getset[] = {
+	{"names", (getter)Register_get_names, NULL, drgn_Register_names_DOC},
 	{},
 };
 
-PyStructSequence_Desc Register_desc = {
-	"Register",
-	drgn_Register_DOC,
-	Register_fields,
-	2,
+PyTypeObject Register_type = {
+	PyVarObject_HEAD_INIT(NULL, 0)
+	.tp_name = "_drgn.Register",
+	.tp_basicsize = sizeof(Register),
+	.tp_repr = (reprfunc)Register_repr,
+	.tp_flags = Py_TPFLAGS_DEFAULT,
+	.tp_doc = drgn_Register_DOC,
+	.tp_richcompare = (richcmpfunc)Register_richcompare,
+	.tp_getset = Register_getset,
 };
-
-PyTypeObject Register_type;
