@@ -2,11 +2,11 @@
 // SPDX-License-Identifier: GPL-3.0+
 
 #include <assert.h>
-#include <byteswap.h>
+#include <endian.h>
 #include <string.h>
 
-#include "internal.h"
 #include "serialize.h"
+#include "minmax.h"
 
 void serialize_bits(void *buf, uint64_t bit_offset, uint64_t uvalue,
 		    uint8_t bit_size, bool little_endian)
