@@ -4,7 +4,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 
-#include "internal.h"
+#include "bitops.h"
 #include "string_builder.h"
 
 bool string_builder_finalize(struct string_builder *sb, char **ret)
