@@ -7,6 +7,7 @@
 #include "error.h"
 #include "hash_table.h"
 #include "language.h"
+#include "lazy_object.h"
 #include "program.h"
 #include "type.h"
 #include "util.h"
@@ -173,60 +174,80 @@ DEFINE_HASH_TABLE_FUNCTIONS(drgn_member_map, drgn_member_key_hash_pair,
 
 DEFINE_HASH_TABLE_FUNCTIONS(drgn_type_set, ptr_key_hash_pair, scalar_key_eq)
 
-struct drgn_error *drgn_lazy_type_evaluate(struct drgn_lazy_type *lazy_type,
-					   struct drgn_qualified_type *ret)
+LIBDRGN_PUBLIC struct drgn_error *
+drgn_member_object(struct drgn_type_member *member,
+		   const struct drgn_object **ret)
 {
-	if (drgn_lazy_type_is_evaluated(lazy_type)) {
-		ret->type = lazy_type->type;
-		ret->qualifiers = lazy_type->qualifiers;
-	} else {
-		struct drgn_type_thunk *thunk_ptr = lazy_type->thunk;
-		struct drgn_type_thunk thunk = *thunk_ptr;
-		struct drgn_error *err = thunk.evaluate_fn(thunk_ptr, ret);
-		if (err)
-			return err;
-		if (drgn_type_program(ret->type) != thunk.prog) {
-			return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
-						 "type is from different program");
-		}
-		drgn_lazy_type_init_evaluated(lazy_type, ret->type,
-					      ret->qualifiers);
-		thunk.free_fn(thunk_ptr);
-	}
-	return NULL;
+	struct drgn_error *err = drgn_lazy_object_evaluate(&member->object);
+	if (!err)
+		*ret = &member->object.obj;
+	return err;
 }
 
-void drgn_lazy_type_deinit(struct drgn_lazy_type *lazy_type)
+LIBDRGN_PUBLIC struct drgn_error *
+drgn_member_type(struct drgn_type_member *member,
+		 struct drgn_qualified_type *type_ret,
+		 uint64_t *bit_field_size_ret)
 {
-	if (!drgn_lazy_type_is_evaluated(lazy_type))
-		drgn_type_thunk_free(lazy_type->thunk);
-}
-
-static inline struct drgn_error *
-drgn_lazy_type_check_prog(struct drgn_lazy_type *lazy_type,
-			  struct drgn_program *prog)
-{
-	if ((drgn_lazy_type_is_evaluated(lazy_type) ?
-	     drgn_type_program(lazy_type->type) :
-	     lazy_type->thunk->prog) != prog) {
-		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
-					 "type is from different program");
+	struct drgn_error *err = drgn_lazy_object_evaluate(&member->object);
+	if (err)
+		return err;
+	*type_ret = drgn_object_qualified_type(&member->object.obj);
+	if (bit_field_size_ret) {
+		if (member->object.obj.is_bit_field)
+			*bit_field_size_ret = member->object.obj.bit_size;
+		else
+			*bit_field_size_ret = 0;
 	}
 	return NULL;
 }
 
 LIBDRGN_PUBLIC struct drgn_error *
-drgn_member_type(struct drgn_type_member *member,
-		 struct drgn_qualified_type *ret)
+drgn_parameter_default_argument(struct drgn_type_parameter *parameter,
+				const struct drgn_object **ret)
 {
-	return drgn_lazy_type_evaluate(&member->type, ret);
+	struct drgn_error *err =
+		drgn_lazy_object_evaluate(&parameter->default_argument);
+	if (!err)
+		*ret = &parameter->default_argument.obj;
+	return err;
 }
 
 LIBDRGN_PUBLIC struct drgn_error *
 drgn_parameter_type(struct drgn_type_parameter *parameter,
 		    struct drgn_qualified_type *ret)
 {
-	return drgn_lazy_type_evaluate(&parameter->type, ret);
+	struct drgn_error *err =
+		drgn_lazy_object_evaluate(&parameter->default_argument);
+	if (!err)
+		*ret = drgn_object_qualified_type(&parameter->default_argument.obj);
+	return err;
+}
+
+LIBDRGN_PUBLIC struct drgn_error *
+drgn_template_parameter_type(struct drgn_type_template_parameter *parameter,
+			     struct drgn_qualified_type *ret)
+{
+	struct drgn_error *err =
+		drgn_lazy_object_evaluate(&parameter->argument);
+	if (!err)
+		*ret = drgn_object_qualified_type(&parameter->argument.obj);
+	return err;
+}
+
+LIBDRGN_PUBLIC struct drgn_error *
+drgn_template_parameter_object(struct drgn_type_template_parameter *parameter,
+			       const struct drgn_object **ret)
+{
+	struct drgn_error *err =
+		drgn_lazy_object_evaluate(&parameter->argument);
+	if (err)
+		return err;
+	if (parameter->argument.obj.kind == DRGN_OBJECT_ABSENT)
+		*ret = NULL;
+	else
+		*ret = &parameter->argument.obj;
+	return NULL;
 }
 
 static struct hash_pair
@@ -234,14 +255,8 @@ drgn_type_dedupe_hash_pair(struct drgn_type * const *entry)
 {
 	struct drgn_type *type = *entry;
 	size_t hash = hash_combine(drgn_type_kind(type),
-				   (uintptr_t)drgn_type_language(type));
-	/*
-	 * We don't dedupe complete compound or enumerated types, and typedefs
-	 * inherit is_complete from the aliased type, so is_complete can only
-	 * differ for otherwise equal array types. We implicitly include that in
-	 * the hash with the is_complete check below, so we don't need to hash
-	 * it explicitly.
-	 */
+				   drgn_type_is_complete(type));
+	hash = hash_combine(hash, (uintptr_t)drgn_type_language(type));
 	if (drgn_type_has_name(type))
 		hash = hash_combine(hash, hash_c_string(drgn_type_name(type)));
 	if (drgn_type_has_size(type))
@@ -257,8 +272,10 @@ drgn_type_dedupe_hash_pair(struct drgn_type * const *entry)
 		hash = hash_combine(hash, (uintptr_t)qualified_type.type);
 		hash = hash_combine(hash, qualified_type.qualifiers);
 	}
-	if (drgn_type_has_length(type) && drgn_type_is_complete(type))
+	if (drgn_type_has_length(type))
 		hash = hash_combine(hash, drgn_type_length(type));
+	if (drgn_type_has_is_variadic(type))
+		hash = hash_combine(hash, drgn_type_is_variadic(type));
 	return hash_pair_from_avalanching_hash(hash);
 }
 
@@ -269,8 +286,8 @@ static bool drgn_type_dedupe_eq(struct drgn_type * const *entry_a,
 	struct drgn_type *b = *entry_b;
 
 	if (drgn_type_kind(a) != drgn_type_kind(b) ||
-	    drgn_type_language(a) != drgn_type_language(b) ||
-	    drgn_type_is_complete(a) != drgn_type_is_complete(b))
+	    drgn_type_is_complete(a) != drgn_type_is_complete(b) ||
+	    drgn_type_language(a) != drgn_type_language(b))
 		return false;
 	if (drgn_type_has_name(a) &&
 	    strcmp(drgn_type_name(a), drgn_type_name(b)) != 0)
@@ -296,13 +313,15 @@ static bool drgn_type_dedupe_eq(struct drgn_type * const *entry_a,
 	if (drgn_type_has_length(a) &&
 	    drgn_type_length(a) != drgn_type_length(b))
 		return false;
+	if (drgn_type_has_is_variadic(a) &&
+	    drgn_type_is_variadic(a) != drgn_type_is_variadic(b))
+		return false;
 	return true;
 }
 
 /*
- * We don't deduplicate complete compound types, complete enumerated types, or
- * function types, so the hash and comparison functions ignore members,
- * enumerators, parameters, and is_variadic.
+ * We don't deduplicate types with members, parameters, template parameters, or
+ * enumerators, so the hash and comparison functions ignore those.
  */
 DEFINE_HASH_TABLE_FUNCTIONS(drgn_dedupe_type_set, drgn_type_dedupe_hash_pair,
 			    drgn_type_dedupe_eq)
@@ -454,6 +473,43 @@ struct drgn_error *drgn_complex_type_create(struct drgn_program *prog,
 	return find_or_create_type(&key, ret);
 }
 
+DEFINE_VECTOR_FUNCTIONS(drgn_type_template_parameter_vector)
+
+static void
+drgn_template_parameters_builder_init(struct drgn_template_parameters_builder *builder,
+				      struct drgn_program *prog)
+{
+	builder->prog = prog;
+	drgn_type_template_parameter_vector_init(&builder->parameters);
+}
+
+static void
+drgn_template_parameters_builder_deinit(struct drgn_template_parameters_builder *builder)
+{
+	for (size_t i = 0; i < builder->parameters.size; i++)
+		drgn_lazy_object_deinit(&builder->parameters.data[i].argument);
+	drgn_type_template_parameter_vector_deinit(&builder->parameters);
+}
+
+struct drgn_error *
+drgn_template_parameters_builder_add(struct drgn_template_parameters_builder *builder,
+				     const union drgn_lazy_object *argument,
+				     const char *name, bool is_default)
+{
+	struct drgn_error *err = drgn_lazy_object_check_prog(argument,
+							     builder->prog);
+	if (err)
+		return err;
+	struct drgn_type_template_parameter *parameter =
+		drgn_type_template_parameter_vector_append_entry(&builder->parameters);
+	if (!parameter)
+		return &drgn_enomem;
+	parameter->argument = *argument;
+	parameter->name = name;
+	parameter->is_default = is_default;
+	return NULL;
+}
+
 DEFINE_VECTOR_FUNCTIONS(drgn_type_member_vector)
 
 void drgn_compound_type_builder_init(struct drgn_compound_type_builder *builder,
@@ -463,7 +519,7 @@ void drgn_compound_type_builder_init(struct drgn_compound_type_builder *builder,
 	assert(kind == DRGN_TYPE_STRUCT ||
 	       kind == DRGN_TYPE_UNION ||
 	       kind == DRGN_TYPE_CLASS);
-	builder->prog = prog;
+	drgn_template_parameters_builder_init(&builder->template_builder, prog);
 	builder->kind = kind;
 	drgn_type_member_vector_init(&builder->members);
 }
@@ -472,83 +528,99 @@ void
 drgn_compound_type_builder_deinit(struct drgn_compound_type_builder *builder)
 {
 	for (size_t i = 0; i < builder->members.size; i++)
-		drgn_lazy_type_deinit(&builder->members.data[i].type);
+		drgn_lazy_object_deinit(&builder->members.data[i].object);
 	drgn_type_member_vector_deinit(&builder->members);
+	drgn_template_parameters_builder_deinit(&builder->template_builder);
 }
 
 struct drgn_error *
 drgn_compound_type_builder_add_member(struct drgn_compound_type_builder *builder,
-				      struct drgn_lazy_type type,
-				      const char *name, uint64_t bit_offset,
-				      uint64_t bit_field_size)
+				      const union drgn_lazy_object *object,
+				      const char *name, uint64_t bit_offset)
 {
-	struct drgn_error *err = drgn_lazy_type_check_prog(&type,
-							   builder->prog);
+	struct drgn_error *err =
+		drgn_lazy_object_check_prog(object,
+					    builder->template_builder.prog);
 	if (err)
 		return err;
 	struct drgn_type_member *member =
 		drgn_type_member_vector_append_entry(&builder->members);
 	if (!member)
 		return &drgn_enomem;
-	member->type = type;
+	member->object = *object;
 	member->name = name;
 	member->bit_offset = bit_offset;
-	member->bit_field_size = bit_field_size;
 	return NULL;
 }
 
 struct drgn_error *
 drgn_compound_type_create(struct drgn_compound_type_builder *builder,
-			  const char *tag, uint64_t size,
+			  const char *tag, uint64_t size, bool is_complete,
 			  const struct drgn_language *lang,
 			  struct drgn_type **ret)
 {
+	struct drgn_error *err;
+	struct drgn_program *prog = builder->template_builder.prog;
+
+	if (!is_complete) {
+		if (builder->members.size) {
+			return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
+						 "incomplete type must not have members");
+		}
+		if (size != 0) {
+			return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
+						 "size of incomplete type must be zero");
+		}
+	}
+
+	if (!builder->members.size &&
+	    !builder->template_builder.parameters.size) {
+		struct drgn_type key = {
+			{
+				.kind = builder->kind,
+				.is_complete = is_complete,
+				.primitive = DRGN_NOT_PRIMITIVE_TYPE,
+				.tag = tag,
+				.size = size,
+				.program = prog,
+				.language =
+					lang ? lang : drgn_program_language(prog),
+			}
+		};
+		err = find_or_create_type(&key, ret);
+		if (!err)
+			drgn_type_member_vector_deinit(&builder->members);
+		return err;
+	}
+
 	struct drgn_type *type = malloc(sizeof(*type));
 	if (!type)
 		return &drgn_enomem;
-	if (!drgn_typep_vector_append(&builder->prog->created_types, &type)) {
+	if (!drgn_typep_vector_append(&prog->created_types, &type)) {
 		free(type);
 		return &drgn_enomem;
 	}
 
 	drgn_type_member_vector_shrink_to_fit(&builder->members);
+	drgn_type_template_parameter_vector_shrink_to_fit(&builder->template_builder.parameters);
 
 	type->_private.kind = builder->kind;
-	type->_private.is_complete = true;
+	type->_private.is_complete = is_complete;
 	type->_private.primitive = DRGN_NOT_PRIMITIVE_TYPE;
 	type->_private.tag = tag;
 	type->_private.size = size;
 	type->_private.members = builder->members.data;
 	type->_private.num_members = builder->members.size;
-	type->_private.program = builder->prog;
-	type->_private.language =
-		lang ? lang : drgn_program_language(builder->prog);
+	type->_private.template_parameters =
+		builder->template_builder.parameters.data;
+	type->_private.num_template_parameters =
+		builder->template_builder.parameters.size;
+	type->_private.program = prog;
+	type->_private.language = lang ? lang : drgn_program_language(prog);
 	*ret = type;
 	return NULL;
 }
 
-struct drgn_error *
-drgn_incomplete_compound_type_create(struct drgn_program *prog,
-				     enum drgn_type_kind kind, const char *tag,
-				     const struct drgn_language *lang,
-				     struct drgn_type **ret)
-{
-	assert(kind == DRGN_TYPE_STRUCT ||
-	       kind == DRGN_TYPE_UNION ||
-	       kind == DRGN_TYPE_CLASS);
-	struct drgn_type key = {
-		{
-			.kind = kind,
-			.is_complete = false,
-			.primitive = DRGN_NOT_PRIMITIVE_TYPE,
-			.tag = tag,
-			.program = prog,
-			.language = lang ? lang : drgn_program_language(prog),
-		}
-	};
-	return find_or_create_type(&key, ret);
-}
-
 DEFINE_VECTOR_FUNCTIONS(drgn_type_enumerator_vector)
 
 void drgn_enum_type_builder_init(struct drgn_enum_type_builder *builder,
@@ -595,6 +667,8 @@ struct drgn_error *drgn_enum_type_create(struct drgn_enum_type_builder *builder,
 					 const struct drgn_language *lang,
 					 struct drgn_type **ret)
 {
+	struct drgn_error *err;
+
 	if (drgn_type_program(compatible_type) != builder->prog) {
 		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
 					 "type is from different program");
@@ -604,6 +678,25 @@ struct drgn_error *drgn_enum_type_create(struct drgn_enum_type_builder *builder,
 					 "compatible type of enum type must be integer type");
 	}
 
+	if (!builder->enumerators.size) {
+		struct drgn_type key = {
+			{
+				.kind = DRGN_TYPE_ENUM,
+				.is_complete = true,
+				.primitive = DRGN_NOT_PRIMITIVE_TYPE,
+				.tag = tag,
+				.type = compatible_type,
+				.program = builder->prog,
+				.language =
+					lang ? lang : drgn_program_language(builder->prog),
+			}
+		};
+		err = find_or_create_type(&key, ret);
+		if (!err)
+			drgn_type_enumerator_vector_deinit(&builder->enumerators);
+		return err;
+	}
+
 	struct drgn_type *type = malloc(sizeof(*type));
 	if (!type)
 		return &drgn_enomem;
@@ -763,7 +856,7 @@ DEFINE_VECTOR_FUNCTIONS(drgn_type_parameter_vector)
 void drgn_function_type_builder_init(struct drgn_function_type_builder *builder,
 				     struct drgn_program *prog)
 {
-	builder->prog = prog;
+	drgn_template_parameters_builder_init(&builder->template_builder, prog);
 	drgn_type_parameter_vector_init(&builder->parameters);
 }
 
@@ -771,24 +864,26 @@ void
 drgn_function_type_builder_deinit(struct drgn_function_type_builder *builder)
 {
 	for (size_t i = 0; i < builder->parameters.size; i++)
-		drgn_lazy_type_deinit(&builder->parameters.data[i].type);
+		drgn_lazy_object_deinit(&builder->parameters.data[i].default_argument);
 	drgn_type_parameter_vector_deinit(&builder->parameters);
+	drgn_template_parameters_builder_deinit(&builder->template_builder);
 }
 
 struct drgn_error *
 drgn_function_type_builder_add_parameter(struct drgn_function_type_builder *builder,
-					 struct drgn_lazy_type type,
+					 const union drgn_lazy_object *default_argument,
 					 const char *name)
 {
-	struct drgn_error *err = drgn_lazy_type_check_prog(&type,
-							   builder->prog);
+	struct drgn_error *err =
+		drgn_lazy_object_check_prog(default_argument,
+					    builder->template_builder.prog);
 	if (err)
 		return err;
 	struct drgn_type_parameter *parameter =
 		drgn_type_parameter_vector_append_entry(&builder->parameters);
 	if (!parameter)
 		return &drgn_enomem;
-	parameter->type = type;
+	parameter->default_argument = *default_argument;
 	parameter->name = name;
 	return NULL;
 }
@@ -799,20 +894,45 @@ drgn_function_type_create(struct drgn_function_type_builder *builder,
 			  bool is_variadic, const struct drgn_language *lang,
 			  struct drgn_type **ret)
 {
-	if (drgn_type_program(return_type.type) != builder->prog) {
+	struct drgn_error *err;
+	struct drgn_program *prog = builder->template_builder.prog;
+
+	if (drgn_type_program(return_type.type) != prog) {
 		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
 					 "type is from different program");
 	}
 
+	if (!builder->parameters.size &&
+	    !builder->template_builder.parameters.size) {
+		struct drgn_type key = {
+			{
+				.kind = DRGN_TYPE_FUNCTION,
+				.is_complete = true,
+				.primitive = DRGN_NOT_PRIMITIVE_TYPE,
+				.type = return_type.type,
+				.qualifiers = return_type.qualifiers,
+				.is_variadic = is_variadic,
+				.program = prog,
+				.language =
+					lang ? lang : drgn_program_language(prog),
+			}
+		};
+		err = find_or_create_type(&key, ret);
+		if (!err)
+			drgn_type_parameter_vector_deinit(&builder->parameters);
+		return err;
+	}
+
 	struct drgn_type *type = malloc(sizeof(*type));
 	if (!type)
 		return &drgn_enomem;
-	if (!drgn_typep_vector_append(&builder->prog->created_types, &type)) {
+	if (!drgn_typep_vector_append(&prog->created_types, &type)) {
 		free(type);
 		return &drgn_enomem;
 	}
 
 	drgn_type_parameter_vector_shrink_to_fit(&builder->parameters);
+	drgn_type_template_parameter_vector_shrink_to_fit(&builder->template_builder.parameters);
 
 	type->_private.kind = DRGN_TYPE_FUNCTION;
 	type->_private.is_complete = true;
@@ -822,326 +942,16 @@ drgn_function_type_create(struct drgn_function_type_builder *builder,
 	type->_private.parameters = builder->parameters.data;
 	type->_private.num_parameters = builder->parameters.size;
 	type->_private.is_variadic = is_variadic;
-	type->_private.program = builder->prog;
-	type->_private.language =
-		lang ? lang : drgn_program_language(builder->prog);
+	type->_private.template_parameters =
+		builder->template_builder.parameters.data;
+	type->_private.num_template_parameters =
+		builder->template_builder.parameters.size;
+	type->_private.program = prog;
+	type->_private.language = lang ? lang : drgn_program_language(prog);
 	*ret = type;
 	return NULL;
 }
 
-struct drgn_type_pair {
-	struct drgn_type *a;
-	struct drgn_type *b;
-};
-
-static struct hash_pair
-drgn_type_pair_hash_pair(const struct drgn_type_pair *pair)
-{
-	return hash_pair_from_avalanching_hash(hash_combine((uintptr_t)pair->a,
-							    (uintptr_t)pair->b));
-}
-
-static bool drgn_type_pair_eq(const struct drgn_type_pair *a,
-			      const struct drgn_type_pair *b)
-{
-	return a->a == b->a && a->b == b->b;
-}
-
-DEFINE_HASH_SET(drgn_type_pair_set, struct drgn_type_pair,
-		drgn_type_pair_hash_pair, drgn_type_pair_eq)
-
-static struct drgn_error *drgn_type_eq_impl(struct drgn_type *a,
-					    struct drgn_type *b,
-					    struct drgn_type_pair_set *cache,
-					    int *depth, bool *ret);
-
-static struct drgn_error *
-drgn_qualified_type_eq_impl(struct drgn_qualified_type *a,
-			    struct drgn_qualified_type *b,
-			    struct drgn_type_pair_set *cache, int *depth,
-			    bool *ret)
-{
-	if (a->qualifiers != b->qualifiers) {
-		*ret = false;
-		return NULL;
-	}
-	return drgn_type_eq_impl(a->type, b->type, cache, depth, ret);
-}
-
-static struct drgn_error *drgn_type_members_eq(struct drgn_type *a,
-					       struct drgn_type *b,
-					       struct drgn_type_pair_set *cache,
-					       int *depth, bool *ret)
-{
-	struct drgn_type_member *members_a, *members_b;
-	size_t num_a, num_b, i;
-
-	num_a = drgn_type_num_members(a);
-	num_b = drgn_type_num_members(b);
-	if (num_a != num_b)
-		goto out_false;
-
-	members_a = drgn_type_members(a);
-	members_b = drgn_type_members(b);
-	for (i = 0; i < num_a; i++) {
-		struct drgn_qualified_type type_a, type_b;
-		struct drgn_error *err;
-
-		if (members_a[i].bit_offset != members_b[i].bit_offset ||
-		    members_a[i].bit_field_size != members_b[i].bit_field_size)
-			goto out_false;
-		if ((members_a[i].name || members_b[i].name) &&
-		    (!members_a[i].name || !members_b[i].name ||
-		     strcmp(members_a[i].name, members_b[i].name) != 0))
-			goto out_false;
-
-		err = drgn_member_type(&members_a[i], &type_a);
-		if (err)
-			return err;
-		err = drgn_member_type(&members_b[i], &type_b);
-		if (err)
-			return err;
-		err = drgn_qualified_type_eq_impl(&type_a, &type_b, cache,
-						  depth, ret);
-		if (err)
-			return err;
-		if (!*ret)
-			return NULL;
-	}
-
-	*ret = true;
-	return NULL;
-
-out_false:
-	*ret = false;
-	return NULL;
-}
-
-static bool drgn_type_enumerators_eq(struct drgn_type *a, struct drgn_type *b)
-{
-	const struct drgn_type_enumerator *enumerators_a, *enumerators_b;
-	size_t num_a, num_b, i;
-
-	num_a = drgn_type_num_enumerators(a);
-	num_b = drgn_type_num_enumerators(b);
-	if (num_a != num_b)
-		return false;
-
-	enumerators_a = drgn_type_enumerators(a);
-	enumerators_b = drgn_type_enumerators(b);
-	for (i = 0; i < num_a; i++) {
-		if (strcmp(enumerators_a[i].name, enumerators_b[i].name) != 0)
-			return false;
-		if (enumerators_a[i].uvalue != enumerators_b[i].uvalue)
-			return false;
-	}
-	return true;
-}
-
-static struct drgn_error *
-drgn_type_parameters_eq(struct drgn_type *a, struct drgn_type *b,
-			struct drgn_type_pair_set *cache, int *depth, bool *ret)
-{
-	struct drgn_type_parameter *parameters_a, *parameters_b;
-	size_t num_a, num_b, i;
-
-	num_a = drgn_type_num_parameters(a);
-	num_b = drgn_type_num_parameters(b);
-	if (num_a != num_b)
-		goto out_false;
-
-	parameters_a = drgn_type_parameters(a);
-	parameters_b = drgn_type_parameters(b);
-	for (i = 0; i < num_a; i++) {
-		struct drgn_qualified_type type_a, type_b;
-		struct drgn_error *err;
-
-		if (!parameters_a[i].name && !parameters_b[i].name)
-			continue;
-		if (!parameters_a[i].name || !parameters_b[i].name ||
-		    strcmp(parameters_a[i].name, parameters_b[i].name) != 0)
-			goto out_false;
-
-		err = drgn_parameter_type(&parameters_a[i], &type_a);
-		if (err)
-			return err;
-		err = drgn_parameter_type(&parameters_b[i], &type_b);
-		if (err)
-			return err;
-		err = drgn_qualified_type_eq_impl(&type_a, &type_b, cache,
-						  depth, ret);
-		if (err)
-			return err;
-		if (!*ret)
-			return NULL;
-	}
-
-	*ret = true;
-	return NULL;
-
-out_false:
-	*ret = false;
-	return NULL;
-}
-
-static struct drgn_error *drgn_type_eq_impl(struct drgn_type *a,
-					    struct drgn_type *b,
-					    struct drgn_type_pair_set *cache,
-					    int *depth, bool *ret)
-{
-	struct drgn_error *err;
-
-	if (*depth >= 1000) {
-		return drgn_error_create(DRGN_ERROR_RECURSION,
-					 "maximum type comparison depth exceeded");
-	}
-
-	if (a == b) {
-		*ret = true;
-		return NULL;
-	}
-	if (!a || !b) {
-		*ret = false;
-		return NULL;
-	}
-
-	/*
-	 * Cache this comparison so that we don't do it again. We insert the
-	 * cache entry before doing the comparison in order to break cycles.
-	 */
-	struct drgn_type_pair pair = { a, b };
-	struct hash_pair hp = drgn_type_pair_set_hash(&pair);
-	switch (drgn_type_pair_set_insert_hashed(cache, &pair, hp, NULL)) {
-	case 1:
-		/* These types haven't been compared yet. */
-		break;
-	case 0:
-		/*
-		 * These types have either already been compared, in which case
-		 * they must be equal (otherwise we would've returned false
-		 * immediately), or they are currently being compared, in which
-		 * case they are equal as long as everything we compare after
-		 * this is equal.
-		 */
-		*ret = true;
-		return NULL;
-	case -1:
-		return &drgn_enomem;
-	}
-	(*depth)++;
-
-	if (drgn_type_kind(a) != drgn_type_kind(b) ||
-	    drgn_type_is_complete(a) != drgn_type_is_complete(b))
-		goto out_false;
-
-	switch (drgn_type_kind(a)) {
-	/*
-	 * This types are uniquely deduplicated, so if their pointers did not
-	 * compare equal then they are not equal.
-	 */
-	case DRGN_TYPE_VOID:
-	case DRGN_TYPE_INT:
-	case DRGN_TYPE_BOOL:
-	case DRGN_TYPE_FLOAT:
-	case DRGN_TYPE_COMPLEX:
-		goto out_false;
-	/* These types are uniquely deduplicated only if incomplete. */
-	case DRGN_TYPE_STRUCT:
-	case DRGN_TYPE_UNION:
-	case DRGN_TYPE_CLASS:
-	case DRGN_TYPE_ENUM:
-		if (!drgn_type_is_complete(a))
-			goto out_false;
-		break;
-	/*
-	 * These types are not uniquely deduplicated because they can refer to
-	 * types that are not deduplicated.
-	 */
-	case DRGN_TYPE_TYPEDEF:
-	case DRGN_TYPE_POINTER:
-	case DRGN_TYPE_ARRAY:
-	case DRGN_TYPE_FUNCTION:
-		break;
-	}
-
-	if (drgn_type_language(a) != drgn_type_language(b))
-		goto out_false;
-
-	if (drgn_type_has_name(a) &&
-	    strcmp(drgn_type_name(a), drgn_type_name(b)) != 0)
-		goto out_false;
-	if (drgn_type_has_tag(a)) {
-		const char *tag_a = drgn_type_tag(a);
-		const char *tag_b = drgn_type_tag(b);
-		if ((!tag_a != !tag_b) || (tag_a && strcmp(tag_a, tag_b) != 0))
-			goto out_false;
-	}
-	if (drgn_type_has_size(a) && drgn_type_size(a) != drgn_type_size(b))
-		goto out_false;
-	if (drgn_type_has_length(a) &&
-	    drgn_type_length(a) != drgn_type_length(b))
-		goto out_false;
-	assert(!drgn_type_has_is_signed(a));
-	if (drgn_type_has_type(a)) {
-		struct drgn_qualified_type type_a = drgn_type_type(a);
-		struct drgn_qualified_type type_b = drgn_type_type(b);
-		err = drgn_qualified_type_eq_impl(&type_a, &type_b, cache,
-						  depth, ret);
-		if (err || !*ret)
-			goto out;
-	}
-	if (drgn_type_has_members(a)) {
-		err = drgn_type_members_eq(a, b, cache, depth, ret);
-		if (err || !*ret)
-			goto out;
-	}
-	if (drgn_type_has_enumerators(a) && !drgn_type_enumerators_eq(a, b))
-		goto out_false;
-	if (drgn_type_has_parameters(a)) {
-		err = drgn_type_parameters_eq(a, b, cache, depth, ret);
-		if (err || !*ret)
-			goto out;
-	}
-	if (drgn_type_has_is_variadic(a) &&
-	    drgn_type_is_variadic(a) != drgn_type_is_variadic(b))
-		goto out_false;
-
-	*ret = true;
-	err = NULL;
-	goto out;
-out_false:
-	*ret = false;
-	err = NULL;
-out:
-	(*depth)--;
-	return err;
-}
-
-LIBDRGN_PUBLIC struct drgn_error *drgn_type_eq(struct drgn_type *a,
-					       struct drgn_type *b, bool *ret)
-{
-	if (drgn_type_program(a) != drgn_type_program(b)) {
-		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
-					 "types are from different programs");
-	}
-	struct drgn_type_pair_set cache = HASH_TABLE_INIT;
-	int depth = 0;
-	struct drgn_error *err = drgn_type_eq_impl(a, b, &cache, &depth, ret);
-	drgn_type_pair_set_deinit(&cache);
-	return err;
-}
-
-LIBDRGN_PUBLIC struct drgn_error *
-drgn_qualified_type_eq(struct drgn_qualified_type a,
-		       struct drgn_qualified_type b, bool *ret)
-{
-	if (a.qualifiers != b.qualifiers) {
-		*ret = false;
-		return NULL;
-	}
-	return drgn_type_eq(a.type, b.type, ret);
-}
-
 LIBDRGN_PUBLIC struct drgn_error *
 drgn_format_type_name(struct drgn_qualified_type qualified_type, char **ret)
 {
@@ -1212,7 +1022,7 @@ LIBDRGN_PUBLIC struct drgn_error *drgn_type_sizeof(struct drgn_type *type,
 					 "cannot get size of incomplete %s type",
 					 drgn_type_kind_spelling[kind]);
 	}
-	switch (kind) {
+	SWITCH_ENUM(kind,
 	case DRGN_TYPE_INT:
 	case DRGN_TYPE_BOOL:
 	case DRGN_TYPE_FLOAT:
@@ -1243,8 +1053,7 @@ LIBDRGN_PUBLIC struct drgn_error *drgn_type_sizeof(struct drgn_type *type,
 	case DRGN_TYPE_FUNCTION:
 		return drgn_error_create(DRGN_ERROR_TYPE,
 					 "cannot get size of function type");
-	}
-	UNREACHABLE();
+	)
 }
 
 struct drgn_error *drgn_type_bit_size(struct drgn_type *type, uint64_t *ret)
@@ -1261,36 +1070,37 @@ struct drgn_error *drgn_type_bit_size(struct drgn_type *type, uint64_t *ret)
 	return NULL;
 }
 
-enum drgn_object_kind drgn_type_object_kind(struct drgn_type *type)
+enum drgn_object_encoding drgn_type_object_encoding(struct drgn_type *type)
 {
-	switch (drgn_type_kind(type)) {
+	SWITCH_ENUM(drgn_type_kind(type),
 	case DRGN_TYPE_INT:
-		return (drgn_type_is_signed(type) ? DRGN_OBJECT_SIGNED :
-			DRGN_OBJECT_UNSIGNED);
+		return (drgn_type_is_signed(type) ?
+			DRGN_OBJECT_ENCODING_SIGNED :
+			DRGN_OBJECT_ENCODING_UNSIGNED);
 	case DRGN_TYPE_BOOL:
 	case DRGN_TYPE_POINTER:
-		return DRGN_OBJECT_UNSIGNED;
+		return DRGN_OBJECT_ENCODING_UNSIGNED;
 	case DRGN_TYPE_FLOAT:
-		return DRGN_OBJECT_FLOAT;
+		return DRGN_OBJECT_ENCODING_FLOAT;
 	case DRGN_TYPE_COMPLEX:
-		return DRGN_OBJECT_BUFFER;
+		return DRGN_OBJECT_ENCODING_BUFFER;
 	case DRGN_TYPE_STRUCT:
 	case DRGN_TYPE_UNION:
 	case DRGN_TYPE_CLASS:
 	case DRGN_TYPE_ARRAY:
-		return (drgn_type_is_complete(type) ? DRGN_OBJECT_BUFFER :
-			DRGN_OBJECT_INCOMPLETE_BUFFER);
+		return (drgn_type_is_complete(type) ?
+			DRGN_OBJECT_ENCODING_BUFFER :
+			DRGN_OBJECT_ENCODING_INCOMPLETE_BUFFER);
 	case DRGN_TYPE_ENUM:
 		if (!drgn_type_is_complete(type))
-			return DRGN_OBJECT_INCOMPLETE_INTEGER;
+			return DRGN_OBJECT_ENCODING_INCOMPLETE_INTEGER;
 		/* fallthrough */
 	case DRGN_TYPE_TYPEDEF:
-		return drgn_type_object_kind(drgn_type_type(type).type);
+		return drgn_type_object_encoding(drgn_type_type(type).type);
 	case DRGN_TYPE_VOID:
 	case DRGN_TYPE_FUNCTION:
-		return DRGN_OBJECT_NONE;
-	}
-	UNREACHABLE();
+		return DRGN_OBJECT_ENCODING_NONE;
+	)
 }
 
 struct drgn_error *drgn_type_error(const char *format, struct drgn_type *type)
@@ -1343,22 +1153,6 @@ struct drgn_error *drgn_error_incomplete_type(const char *format,
 	}
 }
 
-struct drgn_error *drgn_error_member_not_found(struct drgn_type *type,
-					       const char *member_name)
-{
-	struct drgn_error *err;
-	struct drgn_qualified_type qualified_type = { type };
-	char *name;
-
-	err = drgn_format_type_name(qualified_type, &name);
-	if (err)
-		return err;
-	err = drgn_error_format(DRGN_ERROR_LOOKUP, "'%s' has no member '%s'",
-				name, member_name);
-	free(name);
-	return err;
-}
-
 void drgn_program_init_types(struct drgn_program *prog)
 {
 	for (size_t i = 0; i < ARRAY_SIZE(prog->void_types); i++) {
@@ -1387,7 +1181,7 @@ void drgn_program_deinit_types(struct drgn_program *prog)
 				drgn_type_members(type);
 			size_t num_members = drgn_type_num_members(type);
 			for (size_t j = 0; j < num_members; j++)
-				drgn_lazy_type_deinit(&members[j].type);
+				drgn_lazy_object_deinit(&members[j].object);
 			free(members);
 		}
 		if (drgn_type_has_enumerators(type))
@@ -1397,7 +1191,7 @@ void drgn_program_deinit_types(struct drgn_program *prog)
 				drgn_type_parameters(type);
 			size_t num_parameters = drgn_type_num_parameters(type);
 			for (size_t j = 0; j < num_parameters; j++)
-				drgn_lazy_type_deinit(&parameters[j].type);
+				drgn_lazy_object_deinit(&parameters[j].default_argument);
 			free(parameters);
 		}
 		free(type);
@@ -1644,10 +1438,11 @@ out:
 }
 
 static struct drgn_error *
-drgn_program_cache_members(struct drgn_program *prog,
-			   struct drgn_type *outer_type,
-			   struct drgn_type *type, uint64_t bit_offset)
+drgn_type_cache_members(struct drgn_type *outer_type,
+			struct drgn_type *type, uint64_t bit_offset)
 {
+	struct drgn_program *prog = drgn_type_program(outer_type);
+
 	if (!drgn_type_has_members(type))
 		return NULL;
 
@@ -1663,11 +1458,9 @@ drgn_program_cache_members(struct drgn_program *prog,
 					.name_len = strlen(member->name),
 				},
 				.value = {
-					.type = &member->type,
+					.member = member,
 					.bit_offset =
 						bit_offset + member->bit_offset,
-					.bit_field_size =
-						member->bit_field_size,
 				},
 			};
 			if (drgn_member_map_insert(&prog->members, &entry,
@@ -1676,13 +1469,14 @@ drgn_program_cache_members(struct drgn_program *prog,
 		} else {
 			struct drgn_qualified_type member_type;
 			struct drgn_error *err = drgn_member_type(member,
-								  &member_type);
+								  &member_type,
+								  NULL);
 			if (err)
 				return err;
-			err = drgn_program_cache_members(prog, outer_type,
-							 member_type.type,
-							 bit_offset +
-							 member->bit_offset);
+			err = drgn_type_cache_members(outer_type,
+						      member_type.type,
+						      bit_offset +
+						      member->bit_offset);
 			if (err)
 				return err;
 		}
@@ -1690,12 +1484,32 @@ drgn_program_cache_members(struct drgn_program *prog,
 	return NULL;
 }
 
-struct drgn_error *drgn_program_find_member(struct drgn_program *prog,
-					    struct drgn_type *type,
-					    const char *member_name,
-					    size_t member_name_len,
-					    struct drgn_member_value **ret)
+LIBDRGN_PUBLIC struct drgn_error *
+drgn_type_offsetof(struct drgn_type *type, const char *member_designator,
+		   uint64_t *ret)
 {
+
+	struct drgn_error *err;
+	const struct drgn_language *lang = drgn_type_language(type);
+	uint64_t bit_offset;
+	err = lang->bit_offset(drgn_type_program(type), type, member_designator,
+			       &bit_offset);
+	if (err)
+		return err;
+	if (bit_offset % 8) {
+		return drgn_error_format(DRGN_ERROR_INVALID_ARGUMENT,
+					 "member is not byte-aligned");
+	}
+	*ret = bit_offset / 8;
+	return NULL;
+}
+
+static struct drgn_error *
+drgn_type_find_member_impl(struct drgn_type *type, const char *member_name,
+			   size_t member_name_len,
+			   struct drgn_member_value **ret)
+{
+	struct drgn_program *prog = drgn_type_program(type);
 	const struct drgn_member_key key = {
 		.type = drgn_underlying_type(type),
 		.name = member_name,
@@ -1725,11 +1539,12 @@ struct drgn_error *drgn_program_find_member(struct drgn_program *prog,
 	}
 	struct hash_pair cached_hp = drgn_type_set_hash(&key.type);
 	if (drgn_type_set_search_hashed(&prog->members_cached, &key.type,
-					cached_hp).entry)
-		return drgn_error_member_not_found(type, member_name);
+					cached_hp).entry) {
+		*ret = NULL;
+		return NULL;
+	}
 
-	struct drgn_error *err = drgn_program_cache_members(prog, key.type,
-							    key.type, 0);
+	struct drgn_error *err = drgn_type_cache_members(key.type, key.type, 0);
 	if (err)
 		return err;
 
@@ -1743,5 +1558,52 @@ struct drgn_error *drgn_program_find_member(struct drgn_program *prog,
 		return NULL;
 	}
 
-	return drgn_error_member_not_found(type, member_name);
+	*ret = NULL;
+	return NULL;
+}
+
+LIBDRGN_PUBLIC struct drgn_error *
+drgn_type_find_member_len(struct drgn_type *type, const char *member_name,
+			  size_t member_name_len,
+			  struct drgn_type_member **member_ret,
+			  uint64_t *bit_offset_ret)
+{
+	struct drgn_error *err;
+	struct drgn_member_value *member;
+	err = drgn_type_find_member_impl(type, member_name, member_name_len,
+					 &member);
+	if (err)
+		return err;
+	if (!member) {
+		struct drgn_qualified_type qualified_type = { type };
+		char *type_name;
+		err = drgn_format_type_name(qualified_type, &type_name);
+		if (err)
+			return err;
+		err = drgn_error_format(DRGN_ERROR_LOOKUP,
+					"'%s' has no member '%.*s'",
+					type_name,
+					member_name_len > INT_MAX ?
+					INT_MAX : (int)member_name_len,
+					member_name);
+		free(type_name);
+		return err;
+	}
+	*member_ret = member->member;
+	*bit_offset_ret = member->bit_offset;
+	return NULL;
+}
+
+LIBDRGN_PUBLIC struct drgn_error *
+drgn_type_has_member_len(struct drgn_type *type, const char *member_name,
+			 size_t member_name_len, bool *ret)
+{
+	struct drgn_error *err;
+	struct drgn_member_value *member;
+	err = drgn_type_find_member_impl(type, member_name, member_name_len,
+					 &member);
+	if (err)
+		return err;
+	*ret = member != NULL;
+	return NULL;
 }
