@@ -29,6 +29,32 @@
 #define UNREACHABLE() assert(!"reachable")
 #endif
 
+/**
+ * Switch statement with an enum controlling expression that must have a case
+ * for every enumeration value and a default case.
+ */
+#define SWITCH_ENUM_DEFAULT(expr, ...) {			\
+	_Pragma("GCC diagnostic push");				\
+	_Pragma("GCC diagnostic error \"-Wswitch-enum\"");	\
+	_Pragma("GCC diagnostic error \"-Wswitch-default\"");	\
+	switch (expr)  {					\
+	__VA_ARGS__						\
+	}							\
+	_Pragma("GCC diagnostic pop");				\
+}
+
+/**
+ * Switch statement with an enum controlling expression that must have a case
+ * for every enumeration value. The expression is assumed to have a valid
+ * enumeration value. Cases which are assumed not to be possible can be placed
+ * at the end of the statement.
+ */
+#define SWITCH_ENUM(expr, ...)		\
+	SWITCH_ENUM_DEFAULT(expr,	\
+	__VA_ARGS__			\
+	default: UNREACHABLE();		\
+	)
+
 #define likely(x) __builtin_expect(!!(x), 1)
 #define unlikely(x) __builtin_expect(!!(x), 0)
 
