@@ -1,6 +1,7 @@
 # Copyright (c) Facebook, Inc. and its affiliates.
 # SPDX-License-Identifier: GPL-3.0+
 
+import itertools
 import unittest
 
 from drgn import Architecture, Platform, PlatformFlags
@@ -13,6 +14,9 @@ class TestPlatform(unittest.TestCase):
 
     def test_registers(self):
         self.assertIn(
-            "rax", (reg.name for reg in Platform(Architecture.X86_64).registers)
+            "rax",
+            itertools.chain.from_iterable(
+                reg.names for reg in Platform(Architecture.X86_64).registers
+            ),
         )
         self.assertEqual(Platform(Architecture.UNKNOWN, PlatformFlags(0)).registers, ())
